# -*- coding: utf-8 -*-

# mapWOC
#
# Description: Standalone- Interface for an NativeHCRunProcess.
#
# Authors:
# Felix Wolfsteller <felix.wolfsteller@intevation.de>
#
# Copyright:
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

import logging
from multiprocessing import Queue
from optparse import OptionParser
import re
import traceback
import time

from mapwoc.lib.hc import HC
from mapwoc.lib.hccontrol import HCControl
from mapwoc.lib.taskqueuepair import TaskQueuePair

from mapwoc.nodectrl.nativehcrunprocess import NativeHCRunProcess


parser = OptionParser()
parser.add_option("-t", "--timeout", dest="timeout", default=60, type="int",
                  help="timeout ('active window') in seconds")
parser.add_option("-l", "--loop", dest="loop", action="store_true",
                  help="Loop forever (defunc).")
parser.add_option("-v", "--verbose", dest="verbose", action="store_true",
                  help="Be verbose.")
parser.add_option("-p", "--poison", dest="poison", action="store_true",
                  help="Give process poison after 5 seconds.")
parser.add_option("-f", "--hcfile", dest="hcfile",
                  help="HCControl file to use.")
(options, args) = parser.parse_args()


if not options.hcfile:
    print "Please provide the hcfile argument."
    exit (1)

if options.loop:
    print "Loop feature is disabled/not implemented."

if options.verbose:
    nodelogger = logging.getLogger('node')
    nodelogger.addHandler(logging.StreamHandler())
    nodelogger.setLevel(logging.DEBUG)

hc = HCControl(filepath=options.hcfile)

try:
    # loop is not a feature of KVMHCRunProcess anymore, implement it here
    io = TaskQueuePair()
    process = NativeHCRunProcess(hc, options.timeout, io, results, './')

    print "Starting Native Dump Process."
    process.start()
    if options.poison:
        time.sleep(5)
        print "putting pill"
        io.add_task("die")

    process.join()

    print "Joined the process."
    try:
        if io.get_results():
            print "Queue not empty."
        while True:
            r = io.get_result(timeout=2)
            print "In Queue: %s." % str(r)
    except:
        pass
except:
    print "Interrupted."
    traceback.print_exc()
    process.cleanup()
    process.terminate()

# TODO Trap SIGINT etc, kill child processes.
exit(0)

